@@ -1,13 +1,13 @@
 ###
-### $Rev: 137 $
-### $Release: 0.6.2 $
-### copyright(c) 2007-2008 kuwata-lab.com all rights reserved.
+### $Release: 0.7.0 $
+### copyright(c) 2007-2009 kuwata-lab.com all rights reserved.
 ###
 
 import unittest
-from test import test_support
-import sys, os, re, time
+import sys, os, re, time, marshal
 from glob import glob
+try:    import cPickle as pickle
+except: import pickle
 
 from testcase_helper import *
 import tenjin
@@ -17,18 +17,18 @@ from tenjin.helpers import *
 
 def _convert_data(data, lang='python'):
     if isinstance(data, dict):
-        for k in data.keys():
+        for k in list(data.keys()):
             v = data[k]
             if k[-1] == '*':
                 assert isinstance(v, dict)
                 data[k[:-1]] = v.get(lang)
-            if isinstance(v, dict) and v.has_key(lang):
+            if isinstance(v, dict) and lang in v:
                 data[k] = v[lang]
             else:
                 _convert_data(v, lang)
     elif isinstance(data, list):
         for k, v in enumerate(data):
-            if isinstance(v, dict) and v.has_key(lang):
+            if isinstance(v, dict) and lang in v:
                 data[k] = v[lang]
             else:
                 _convert_data(v, lang)
@@ -43,7 +43,7 @@ def _remove_files(basenames=[]):
 class EngineTest(unittest.TestCase, TestCaseHelper):
 
     #code = TestCaseHelper.generate_testcode(__file__)
-    #exec code
+    #exec(code)
     datalist = TestCaseHelper.load_testdata(__file__)
     testdata = dict([ (d['name'], d) for d in datalist ])
     _convert_data(testdata, lang='python')
@@ -58,7 +58,7 @@ class EngineTest(unittest.TestCase, TestCaseHelper):
     #def setUp(self):
     #    testdata = EngineTest.testdata['basic']
     #    for hash in testdata['templates']:
-    #        open(hash['filename'], 'w').write(hash['content'])
+    #        write_file(hash['filename'], hash['content'])
 
 
     #def tearDown(self):
@@ -70,15 +70,12 @@ class EngineTest(unittest.TestCase, TestCaseHelper):
 
 
     def _test_basic(self):
-        if not self.is_target():
-            return
-        #
         try:
             testdata = EngineTest.testdata['basic']
             for hash in testdata['templates']:
-                open(hash['filename'], 'w').write(hash['content'])
+                write_file(hash['filename'], hash['content'])
             #
-            testname = self.testname()
+            testname = self._testname()
             lst = testname[len('test_basic'):].split('_')
             action   = lst[1]                # 'list', 'show', 'create', or 'edit'
             shortp   = lst[2] == 'short'     # 'short' or 'long'
@@ -169,8 +166,6 @@ class EngineTest(unittest.TestCase, TestCaseHelper):
 
 
     def test_capture_and_echo(self):
-        if not self.is_target():
-            return
         hash = EngineTest.testdata['test_capture_and_echo']
         layout = hash['layout']
         content = hash['content']
@@ -179,8 +174,8 @@ class EngineTest(unittest.TestCase, TestCaseHelper):
         content_filename = 'user_content.pyhtml'
         context = { 'items': ['AAA', 'BBB', 'CCC'] }
         try:
-            open(layout_filename, 'w').write(layout)
-            open(content_filename, 'w').write(content)
+            write_file(layout_filename, layout)
+            write_file(content_filename, content)
             engine = tenjin.Engine(prefix='user_', postfix='.pyhtml', layout=':layout')
             output = engine.render(':content', context)
             self.assertTextEqual(expected, output)
@@ -189,8 +184,6 @@ class EngineTest(unittest.TestCase, TestCaseHelper):
 
 
     def test_captured_as(self):
-        if not self.is_target():
-            return
         hash = EngineTest.testdata['test_captured_as']
         files = ( ('content.pyhtml',      hash['content']),
                   ('customlayout.pyhtml', hash['customlayout']),
@@ -200,7 +193,7 @@ class EngineTest(unittest.TestCase, TestCaseHelper):
         expected = hash['expected']
         try:
             for filename, content in files:
-                open(filename, 'w').write(content)
+                write_file(filename, content)
             engine = tenjin.Engine(postfix='.pyhtml')
             output = engine.render(':content', context)
             self.assertTextEqual(expected, output)
@@ -209,8 +202,6 @@ class EngineTest(unittest.TestCase, TestCaseHelper):
 
 
     def test_local_layout(self):
-        if not self.is_target():
-            return
         hash = EngineTest.testdata['test_local_layout']
         context = hash['context']
         names = ['layout_html', 'layout_xhtml', 'content_html']
@@ -218,12 +209,12 @@ class EngineTest(unittest.TestCase, TestCaseHelper):
             return 'local_%s.pyhtml' % base
         try:
             for name in names:
-                open(fname(name), 'w').write(hash[name])
+                write_file(fname(name), hash[name])
             engine = tenjin.Engine(prefix='local_', postfix='.pyhtml', layout=':layout_html')
             ##
             def _test(expected, statement):
                 content_html = hash['content_html'] + statement
-                open(fname('content_html'), 'w').write(content_html)
+                write_file(fname('content_html'), content_html)
                 actual = engine.render(':content_html', context)
                 self.assertTextEqual(expected, actual)
             ##
@@ -243,8 +234,6 @@ class EngineTest(unittest.TestCase, TestCaseHelper):
 
 
     def test_cachefile(self):
-        if not self.is_target():
-            return
         data = EngineTest.testdata['test_cachefile']
         filenames = { 'layout': 'layout.pyhtml',
                       'page': 'account_create.pyhtml',
@@ -252,39 +241,93 @@ class EngineTest(unittest.TestCase, TestCaseHelper):
                     }
         expected = data['expected']
         context = { 'params': { } }
+        cache_filenames = ['account_create.pyhtml.cache', 'account_form.pyhtml.cache']
         try:
-            for key, filename in filenames.iteritems():
-                open(filename, 'w').write(data[key])
+            for key, filename in filenames.items():
+                write_file(filename, data[key])
             props = { 'prefix':'account_', 'postfix':'.pyhtml', 'layout':'layout.pyhtml' }
             ## no caching
             props['cache'] = False
             engine = tenjin.Engine(**props)
             output = engine.render(':create', context)
             self.assertTextEqual(expected, output)
-            self.assertFalse(os.path.exists('account_create.pyhtml.cache'))
-            self.assertFalse(os.path.exists('account_form.pyhtml.cache'))
-            ## bytecode caching
+            for fname in cache_filenames: self.assertNotExist(fname)
+            ## marshal caching
             props['cache'] = True
             engine = tenjin.Engine(**props)
             output = engine.render(':create', context)
             self.assertTextEqual(expected, output)
-            self.assertTrue(os.path.exists('account_create.pyhtml.cache'))
-            self.assertTrue(os.path.exists('account_form.pyhtml.cache'))
+            if   python2:  nullbyte = '\0'
+            elif python3:  nullbyte = '\0'.encode('ascii')
+            for fname in cache_filenames:
+                self.assertExists(fname)                         # file created?
+                s = read_file(fname, 'rb')                       # read binary file
+                self.assertTrue(s.find(nullbyte) >= 0)           # binary file?
+                f = lambda: marshal.load(open(fname, 'rb'))
+                self.assertNotRaise(f)                           # marshal?
+            engine = tenjin.Engine(**props)
+            output = engine.render(':create', context)
+            self.assertTextEqual(expected, output)               # reloadable?
+            #
+            for fname in glob('*.pyhtml.cache'): os.unlink(fname)
+            for fname in cache_filenames:
+                self.assertNotExist(fname)
+            ## pickle caching
+            props['cache'] = tenjin.PickleCacheStorage()
             engine = tenjin.Engine(**props)
             output = engine.render(':create', context)
             self.assertTextEqual(expected, output)
+            if   python2:  nullbyte = '\0'
+            elif python3:  nullbyte = '\0'.encode('ascii')
+            for fname in cache_filenames:
+                self.assertExists(fname)                         # file created?
+                s = read_file(fname, 'rb')                       # read text file
+                if python2:
+                    self.assertTrue(s.find(nullbyte) < 0)        # text file? (pickle protocol ver 2)
+                elif python3:
+                    self.assertTrue(s.find(nullbyte) >= 0)       # binary file? (pickle protocol ver 3)
+                f = lambda: Pickle.load(open(fname, 'rb'))
+                self.assertNotRaise(ValueError, f)               # pickle?
+                pickle.load(open(fname, 'rb'))
+            engine = tenjin.Engine(**props)
+            output = engine.render(':create', context)
+            self.assertTextEqual(expected, output)               # reloadable?
+            #
+            for fname in glob('*.cache'): os.unlink(fname)
+            for fname in cache_filenames:
+                self.assertNotExist(fname)
+            ## text caching
+            props['cache'] = tenjin.TextCacheStorage()
+            engine = tenjin.Engine(**props)
+            output = engine.render(':create', context)
+            self.assertTextEqual(expected, output)
+            if   python2:  nullchar = '\0'
+            elif python3:  nullchar = '\0'
+            for fname in cache_filenames:
+                self.assertExists(fname)                         # file created?
+                s = read_file(fname, 'r')                        # read text file
+                self.assertTrue(s.find(nullchar) < 0)            # text file?
+                #f = lambda: marshal.loads(s)
+                f = lambda: marshal.load(open(fname, 'rb'))
+                if python3:
+                    ex = self.assertRaise(ValueError, f)         # non-marshal?
+                    self.assertEquals("bad marshal data", str(ex))
+                elif python2 and sys.version_info[1] >= 5:
+                    ex = self.assertRaise(EOFError, f)           # non-marshal?
+                    self.assertEquals("EOF read where object expected", str(ex))
+            engine = tenjin.Engine(**props)
+            output = engine.render(':create', context)
+            self.assertTextEqual(expected, output)               # reloadable?
         finally:
             _remove_files(filenames.values())
 
 
     def test_change_layout(self):
-        if not self.is_target():
-            return
         data = EngineTest.testdata['test_change_layout']
         ## setup
         basenames = ['baselayout', 'customlayout', 'content']
         for basename in basenames:
-            open('%s.pyhtml' % basename, 'w').write(data[basename])
+            write_file('%s.pyhtml' % basename, data[basename])
         ## body
         try:
             engine = tenjin.Engine(layout='baselayout.pyhtml')
@@ -297,14 +340,12 @@ class EngineTest(unittest.TestCase, TestCaseHelper):
 
 
     def test_context_scope(self):
-        if not self.is_target():
-            return
         data = EngineTest.testdata['test_context_scope']
         base = data['base']
         part = data['part']
         expected = data['expected']
         for basename in ('base', 'part'):
-            open('%s.pyhtml' % basename, 'w').write(data[basename])
+            write_file('%s.pyhtml' % basename, data[basename])
         #
         try:
             engine = tenjin.Engine()
@@ -317,14 +358,12 @@ class EngineTest(unittest.TestCase, TestCaseHelper):
 
 
     def test_template_args(self):
-        if not self.is_target():
-            return
         data = EngineTest.testdata['test_template_args']
         content = data['content']
         expected = data['expected']
         context = data['context']
         for basename in ('content', ):
-            open('%s.pyhtml' % basename, 'w').write(data[basename])
+            write_file('%s.pyhtml' % basename, data[basename])
         #
         try:
             def f1():
@@ -343,19 +382,16 @@ class EngineTest(unittest.TestCase, TestCaseHelper):
 
 
     def test_cached_contents(self):
-        if not self.is_target():
-            return
         data = EngineTest.testdata['test_cached_contents']
-        from glob import glob
         def _test(filename, cachename, cachemode, input, expected_script, expected_args):
             if input:
-                open(filename, 'w').write(input)
+                write_file(filename, input)
             engine = tenjin.Engine(cache=cachemode)
             t = engine.get_template(filename)
             self.assertEqual(expected_args, t.args)
             self.assertTextEqual(expected_script, t.script)
             import marshal
-            dct = marshal.load(open(engine.cachename(filename), 'rb'))
+            dct = marshal.load(open(filename + '.cache', 'rb'))
             self.assertEqual(expected_args, dct['args'])
             self.assertTextEqual(expected_script, dct['script'])
         ##
@@ -367,27 +403,25 @@ class EngineTest(unittest.TestCase, TestCaseHelper):
             args  = data['args1']
             input = data['input1']
             cachename = filename+'.cache'
-            self.assertFalse(os.path.exists(cachename))
-            _test(filename, cachename, 1, input, script, args)
-            self.assertTrue(os.path.exists(cachename))
-            _test(filename, cachename, 1, None, script, args)
+            self.assertNotExist(cachename)
+            _test(filename, cachename, True, input, script, args)
+            self.assertExists(cachename)
+            _test(filename, cachename, True, None, script, args)
             ## args=[], cache=1
             cachename = filename+'.cache'
             input = data['input2']  # re.sub(r'<\?py #@ARGS.*?\?>\n', '<?py #@ARGS ?>\n', input)
             script = data['script2']  # re.sub(r'#@ARGS.*?\n', '#@ARGS \n', cache)
             args  = data['args2']   # []
             time.sleep(1)
-            #self.assertTrue(os.path.exists(cachename))
-            _test(filename, cachename, 1, input, script, args)
-            #self.assertTrue(os.path.exists(cachename))
-            _test(filename, cachename, 1, None, script, args)
+            #self.assertExists(cachename)
+            _test(filename, cachename, True, input, script, args)
+            #self.assertExists(cachename)
+            _test(filename, cachename, True, None, script, args)
         finally:
             _remove_files(['input.pyhtml'])
 
 
     def _test_template_path(self, keys):
-        if not self.is_target():
-            return
         data = EngineTest.testdata['test_template_path']
         basedir = 'test_templates'
         try:
@@ -397,10 +431,10 @@ class EngineTest(unittest.TestCase, TestCaseHelper):
             d = { 'layout':keys[0], 'body':keys[1], 'footer':keys[2], }
             for key in ('layout', 'body', 'footer'):
                 filename = '%s/common/%s.pyhtml' % (basedir, key)
-                open(filename, 'w').write(data['common_'+key])
+                write_file(filename, data['common_'+key])
                 if d[key] == 'user':
                     filename = '%s/user/%s.pyhtml' % (basedir, key)
-                    open(filename, 'w').write(data['user_'+key])
+                    write_file(filename, data['user_'+key])
             #
             path = [basedir+'/user', basedir+'/common']
             engine = tenjin.Engine(postfix='.pyhtml', path=path, layout=':layout')
@@ -437,8 +471,6 @@ class EngineTest(unittest.TestCase, TestCaseHelper):
 
 
     def test_preprocessor(self):
-        if not self.is_target():
-            return
         data = EngineTest.testdata['test_preprocessor']
         try:
             basenames = ('form', 'create', 'update', 'layout', )
@@ -446,7 +478,7 @@ class EngineTest(unittest.TestCase, TestCaseHelper):
             for name in basenames:
                 filename = 'prep_%s.pyhtml' % name
                 filenames.append(filename)
-                open(filename, 'w').write(data[name])
+                write_file(filename, data[name])
             engine = tenjin.Engine(prefix='prep_', postfix='.pyhtml', layout=':layout', preprocess=True)
             #
             context = {
@@ -481,11 +513,8 @@ class EngineTest(unittest.TestCase, TestCaseHelper):
             _remove_files(filenames)
 
 
-
-def test_main():
-    test_support.run_unittest(EngineTest)
+remove_unmatched_test_methods(EngineTest)
 
 
 if __name__ == '__main__':
-    test_main()
-
+    unittest.main()
