@@ -1,13 +1,12 @@
 # -*- coding: utf-8 -*-
 ###
-### $Rev: 137 $
-### $Release: 0.6.2 $
-### copyright(c) 2007-2008 kuwata-lab.com all rights reserved.
+### $Release: 0.7.0 $
+### copyright(c) 2007-2009 kuwata-lab.com all rights reserved.
 ###
 
 import unittest
-from test import test_support
 import os, traceback
+import yaml
 
 from testcase_helper import *
 import tenjin
@@ -18,9 +17,10 @@ for filename in ['../bin/pytenjin', 'bin/pytenjin']:
     if os.path.exists(filename):
         break
 
+_name_orig = __name__
 __name__ = 'dummy'
-execfile(filename)
-__name__ = '__main__'
+exec(tenjin._read_binary_file(filename).decode('utf-8'))
+__name__ = _name_orig
 
 
 def to_list(value):
@@ -29,6 +29,10 @@ def to_list(value):
     return [value]
 
 
+import sys
+python_version = sys.version.split(' ')[0]
+
+
 INPUT = r"""<ul>
 <?py for item in ['<a&b>', '["c",'+"'d']"]: ?>
   <li>#{item}
@@ -116,8 +120,6 @@ class MainTest(unittest.TestCase, TestCaseHelper):
         pass
 
     def _test(self):
-        if not self.is_target(depth=3): return
-        #
         input     = getattr(self, 'input', '')
         source    = getattr(self, 'source', None)
         expected  = getattr(self, 'expected', None)
@@ -129,14 +131,18 @@ class MainTest(unittest.TestCase, TestCaseHelper):
         context_data = getattr(self, 'context_data', None)
         encoding  = getattr(self, 'encoding', None)
         #
+        if python_version < '2.5':
+            if expected:
+                expected = expected.replace(': unexpected indent', ': invalid syntax')
+        #
         if filename is not False:
             if filename is None:
                 filename = '.test.pyhtml'
-            #open(filename, 'w').write(input)
+            #write_file(filename, input)
             for fname, s in zip(to_list(filename), to_list(input)):
-                if encoding and isinstance(s, unicode):
+                if encoding and isinstance(s, _unicode):
                     s = s.encode(encoding)
-                open(fname, 'w').write(s)
+                write_file(fname, s)
         #
         if isinstance(options, list):
             argv = options
@@ -150,9 +156,9 @@ class MainTest(unittest.TestCase, TestCaseHelper):
         #
         if context_file:
             s = context_data
-            if encoding and instance(s, unicode):
+            if encoding and instance(s, _unicode):
                 s = s.encode(encoding)
-            open(context_file, 'w').write(s)
+            write_file(context_file, s)
         #
         try:
             app = Main(argv)
@@ -161,9 +167,10 @@ class MainTest(unittest.TestCase, TestCaseHelper):
                 def f1():
                     try:
                         output = app.execute()
-                    except Exception, err:
-                        lst[0] = err
-                        raise err
+                    except Exception:
+                        ex = sys.exc_info()[1]
+                        lst[0] = ex
+                        raise ex
                 self.assertRaises(exception, f1)
                 if errormsg:
                     ex = lst[0]
@@ -172,8 +179,9 @@ class MainTest(unittest.TestCase, TestCaseHelper):
                 output = app.execute()
                 #print "*** expected=%s" % repr(expected)
                 #print "*** output=%s" % repr(output)
-                if encoding and isinstance(output, unicode):
-                    output = output.encode(encoding)
+                if python2:
+                    if encoding and isinstance(output, unicode):
+                        output = output.encode(encoding)
                 self.assertTextEqual(expected, output)
         finally:
             try:
@@ -187,12 +195,11 @@ class MainTest(unittest.TestCase, TestCaseHelper):
                 pass
 
     code = TestCaseHelper.generate_testcode(__file__)
-    exec code
+    exec(code)
 
 
 
     def test_help(self):  # -h, --help
-        if not self.is_target(): return
         self.options  = "-h"
         self.input    = ""
         self.expected = Main(['tenjin']).usage('tenjin')
@@ -202,7 +209,6 @@ class MainTest(unittest.TestCase, TestCaseHelper):
         self._test()
 
     def test_version(self):  # -v, --version
-        if not self.is_target(): return
         self.options  = "-v"
         self.input    = ""
         self.expected = Main(['tenjin']).version() + "\n"
@@ -211,7 +217,6 @@ class MainTest(unittest.TestCase, TestCaseHelper):
         self._test()
 
 #    def test_help_and_version(self):  # -hVz
-#        if not self.is_target(): return
 #        self.options  = "-hVc"
 #        self.input    = "<?py foo() ?>"
 #        app = Main(['tenjin'])
@@ -219,7 +224,6 @@ class MainTest(unittest.TestCase, TestCaseHelper):
 #        self._test()
 
     def test_render(self):  # (nothing), -a render
-        if not self.is_target(): return
         self.options  = ""
         self.input    = INPUT
         self.expected = EXECUTED
@@ -228,7 +232,6 @@ class MainTest(unittest.TestCase, TestCaseHelper):
         self._test()
 
     def test_source(self):  # -s, -a convert
-        if not self.is_target(): return
         self.options  = "-s"
         self.input    = INPUT
         self.expected = SOURCE
@@ -237,7 +240,6 @@ class MainTest(unittest.TestCase, TestCaseHelper):
         self._test()
 
     def test_source2(self):  # -s, -aconvert
-        if not self.is_target(): return
         self.options  = "-s"
         n1 = len("<ul>\n")
         n2 = len("</ul>\n")
@@ -251,7 +253,6 @@ class MainTest(unittest.TestCase, TestCaseHelper):
         self._test()
 
     def test_source3(self):  # -sb, -baconvert
-        if not self.is_target(): return
         self.options  = "-sb"
         self.input    = INPUT
         n1 = len("_buf = []; ")
@@ -261,6 +262,43 @@ class MainTest(unittest.TestCase, TestCaseHelper):
         self.options = "-baconvert"
         self._test()
 
+    def test_cache1(self):   # -a cache
+        self.options  = "-a cache"
+        self.input    = (
+            '<?py #@ARGS title, items ?>\n'
+            '<h1>${title}</h1>\n'
+            '<ul>\n'
+            '<?py for item in items: ?>\n'
+            '  <li>${item}</li>\n'
+            '<?py #endfor ?>\n'
+            '</ul>\n'
+            )
+        self.expected = ''
+        script = (
+            "title = _context.get('title'); items = _context.get('items'); \n"
+            "_buf.extend(('''<h1>''', escape(to_str(title)), '''</h1>\n"
+            "<ul>\\n''', ));\n"
+            "for item in items:\n"
+            "    _buf.extend(('''  <li>''', escape(to_str(item)), '''</li>\\n''', ));\n"
+            "#endfor\n"
+            "_buf.extend(('''</ul>\\n''', ));\n"
+            )
+        self.filename = 'test_cache1.pyhtml'
+        cachename = self.filename + '.cache'
+        try:
+            self._test()
+            self.assertExists(cachename)
+            import marshal
+            dct = marshal.load(open(cachename, 'rb'))
+            self.assertTextEqual(['title', 'items'], dct.get('args'))
+            if   python2:  expected = "<type 'code'>"
+            elif python3:  expected = "<class 'code'>"
+            self.assertEquals(expected, str(type(dct.get('bytecode'))))
+            self.assertTextEqual(script, dct.get('script'))
+        finally:
+            if os.path.exists(cachename):
+                os.unlink(cachename)
+
     input_for_retrieve = (
         '<div>\n'
         '<?py if list: ?>\n'
@@ -309,7 +347,6 @@ class MainTest(unittest.TestCase, TestCaseHelper):
         ''))
 
     def test_retrieve(self):  # -S, -a retrieve
-        if not self.is_target(): return
         self.input    = self.input_for_retrieve
         self.expected = self.expected_for_retrieve
         self.options = '-S'
@@ -318,7 +355,6 @@ class MainTest(unittest.TestCase, TestCaseHelper):
         self._test()
 
     def test_retrieve2(self):  # -SU, -SNU
-        if not self.is_target(): return
         expected = '\n'.join((
             '_buf = []; ',
             'if list:',
@@ -361,7 +397,6 @@ class MainTest(unittest.TestCase, TestCaseHelper):
         self._test()
 
     def test_retrieve3(self):  # -SC, -SNC
-        if not self.is_target(): return
         expected = '\n'.join((
             '_buf = []; ',
             'if list:',
@@ -398,7 +433,6 @@ class MainTest(unittest.TestCase, TestCaseHelper):
         self._test()
 
     def test_statements(self):  # -X, -a statements
-        if not self.is_target(): return
         expected = '\n'.join((
             '_buf = []; ',
             'if list:',
@@ -430,7 +464,6 @@ class MainTest(unittest.TestCase, TestCaseHelper):
         self._test()
 
     def test_dump(self):  # -d, -a dump
-        if not self.is_target(): return
         # create cache file
         filename = '_test_dump.pyhtml'
         cachename = filename + '.cache'
@@ -439,7 +472,7 @@ class MainTest(unittest.TestCase, TestCaseHelper):
         self.expected = EXECUTED
         self.options = '-a render --cache=true'
         self._test()
-        self.assertTrue(os.path.exists(cachename))
+        self.assertExists(cachename)
         # dump test
         try:
             self.filename = False
@@ -453,14 +486,12 @@ class MainTest(unittest.TestCase, TestCaseHelper):
             os.unlink(cachename)
 
     def test_indent(self):  # -i2
-        if not self.is_target(): return
         self.options  = "-si2"
         self.input    = INPUT
         self.expected = SOURCE.replace('    _buf', '  _buf')
         self._test()
 
     def test_quiet(self):  # -q, -qasyntax
-        if not self.is_target(): return
         self.options  = "-z"
         input = INPUT
         self.input    = [input, input, input]
@@ -476,7 +507,6 @@ class MainTest(unittest.TestCase, TestCaseHelper):
         self._test()
 
     def test_invalid_options(self):  # -Y, -i, -f, -c, -i foo
-        if not self.is_target(): return
         self.input    = INPUT
         self.expected = ""
         self.exception = CommandOptionError
@@ -509,7 +539,6 @@ class MainTest(unittest.TestCase, TestCaseHelper):
         #
 
     def test_newline(self):
-        if not self.is_target(): return
         self.options  = "-s"
         self.input    = INPUT2
         self.expected = SOURCE2
@@ -519,7 +548,6 @@ class MainTest(unittest.TestCase, TestCaseHelper):
         self._test()
 
     def test_datafile_yaml(self): # -f datafile.yaml
-        if not self.is_target(): return
         context_filename = 'test.datafile.yaml'
         self.options  = "-f " + context_filename
         self.input    = INPUT3
@@ -529,7 +557,6 @@ class MainTest(unittest.TestCase, TestCaseHelper):
         self._test()
 
     def test_datafile_py(self): # -f datafile.py
-        if not self.is_target(): return
         context_filename = 'test.datafile.py'
         self.options  = "-f " + context_filename
         self.input    = INPUT3
@@ -539,7 +566,6 @@ class MainTest(unittest.TestCase, TestCaseHelper):
         self._test()
 
     def test_datafile_error(self):  # -f file.txt, not-a-mapping context data
-        if not self.is_target(): return
         context_filename = 'test.datafile.txt'
         self.options = "-f " + context_filename
         self.exception = CommandOptionError
@@ -558,21 +584,18 @@ class MainTest(unittest.TestCase, TestCaseHelper):
         self._test()
 
     def test_context_yaml(self):  # -c yamlstr
-        if not self.is_target(): return
         self.options = ['-c', '{title: tenjin example, items: [aaa, bbb, ccc]}']
         self.input    = INPUT3
         self.expected = EXECUTED3
         self._test()
 
     def test_context_py(self):  # -c python-code
-        if not self.is_target(): return
         self.options = ['-c', 'title="tenjin example";  items=["aaa", "bbb", "ccc"]']
         self.input    = INPUT3
         self.expected = EXECUTED3
         self._test()
 
     def test_untabify(self):  # -T
-        if not self.is_target(): return
         context_filename = 'test.datafile.yaml'
         self.options  = "-Tf " + context_filename
         self.input    = INPUT3
@@ -583,7 +606,6 @@ class MainTest(unittest.TestCase, TestCaseHelper):
         self._test()
 
     def test_modules(self):  # -r modules
-        if not self.is_target(): return
         self.options  = "--escapefunc=cgi.escape"
         self.input    = INPUT
         self.expected = EXECUTED.replace('&quot;', '"')
@@ -599,41 +621,45 @@ class MainTest(unittest.TestCase, TestCaseHelper):
         self._test()
 
     def test_modules_err(self):  # -r hogeratta
-        if not self.is_target(): return
         self.options = '-r hogeratta'
         self.exception = CommandOptionError
         self.errormsg = '-r hogeratta: module not found.'
         self._test()
 
     def test_escapefunc(self):  # --escapefunc=cgi.escape
-        if not self.is_target(): return
         self.options  = "-s --escapefunc=cgi.escape"
         self.input    = INPUT
         self.expected = SOURCE.replace('escape', 'cgi.escape')
         self._test()
 
     def test_tostrfunc(self):  # --tostrfunc=str
-        if not self.is_target(): return
         self.options  = "-s --tostrfunc=str"
         self.input    = INPUT
         self.expected = SOURCE.replace('to_str', 'str')
         self._test()
 
     def test_preamble(self):  # --preamble --postamble
-        if not self.is_target(): return
         self.options  = ["-s", "--preamble=_buf=list()", "--postamble=return ''.join(_buf)"]
         self.input    = INPUT
         self.expected = SOURCE.replace("_buf = []", "_buf=list()").replace("print ", "return ")
         self._test()
 
     def test_xencoding1(self):  # --encoding=encoding
-        if not self.is_target(): return
-        self.input = """\
+        if python2:
+            self.input = """\
 <?py items=['foo',u'bar',u'日本語'] ?>
 ようこそ
 <?py for item in items: ?>
 * 「${item}」
 <?py #end ?>
+"""
+        elif python3:
+            self.input = """\
+<?py items=['foo','bar','日本語'] ?>
+ようこそ
+<?py for item in items: ?>
+* 「${item}」
+<?py #end ?>
 """
         self.expected = """\
 ようこそ
@@ -647,13 +673,21 @@ class MainTest(unittest.TestCase, TestCaseHelper):
         self._test()
 
     def test_xencoding2(self):  # -k encoding
-        if not self.is_target(): return
-        self.input = """\
+        if python2:
+            self.input = """\
 <?py items=['foo',u'bar','日本語'] ?>
 ようこそ
 <?py for item in items: ?>
 * 「${item}」
 <?py #end ?>
+"""
+        elif python3:
+            self.input = """\
+<?py items=['foo',b'bar','日本語'] ?>
+ようこそ
+<?py for item in items: ?>
+* 「${item}」
+<?py #end ?>
 """
         self.expected = """\
 ようこそ
@@ -671,7 +705,6 @@ class MainTest(unittest.TestCase, TestCaseHelper):
             globals()['tostr'] = tostr_func
 
     def test_template_path(self):  # --path
-        if not self.is_target(): return
         layout = r'''<html>
   <body>
 #{_content}
@@ -704,11 +737,11 @@ class MainTest(unittest.TestCase, TestCaseHelper):
         try:
             os.mkdir("tmpl9")
             os.mkdir("tmpl9/user")
-            open("tmpl9/layout.pyhtml", 'w').write(layout)
-            open("tmpl9/body.pyhtml", 'w').write('')
-            open("tmpl9/footer.pyhtml", 'w').write('')
-            open("tmpl9/user/body.pyhtml", 'w').write(body)
-            open("tmpl9/user/footer.pyhtml", 'w').write(footer)
+            write_file("tmpl9/layout.pyhtml", layout)
+            write_file("tmpl9/body.pyhtml", '')
+            write_file("tmpl9/footer.pyhtml", '')
+            write_file("tmpl9/user/body.pyhtml", body)
+            write_file("tmpl9/user/footer.pyhtml", footer)
             self.options  = "--path=.,tmpl9/user,tmpl9 --postfix=.pyhtml --layout=:layout"
             self.input = "<?py include(':body') ?>"
             self.expected = expected
@@ -720,14 +753,13 @@ class MainTest(unittest.TestCase, TestCaseHelper):
             for d in ['tmpl9/user', 'tmpl9']: os.rmdir(d)
 
     def test_preprocess1(self):  # -P, -a preprocess, --preprocess
-        if not self.is_target(): return
         input = '''\
 <?PY states = { "CA": "California", ?>
 <?PY            "NY": "New York", ?>
 <?PY            "FL": "Florida", } ?>
 <?PY # ?>
 <?py chk = { params['state']: ' checked="checked"' } ?>
-<?PY codes = states.keys() ?>
+<?PY codes = list(states.keys()) ?>
 <?PY codes.sort() ?>
 <select name="state">
   <option value="">-</option>
@@ -769,16 +801,8 @@ class MainTest(unittest.TestCase, TestCaseHelper):
             pass
 
 
-name = os.environ.get('TEST')
-if name:
-    for m in dir(MainTest):
-        if m.startswith('test_') and m != 'test_'+name:
-            delattr(MainTest, m)
-
-
-def test_main():
-    test_support.run_unittest(MainTest)
+remove_unmatched_test_methods(MainTest)
 
 
 if __name__ == '__main__':
-    test_main()
+    unittest.main()
