@@ -1,11 +1,9 @@
 ###
-### $Rev: 137 $
-### $Release: 0.6.2 $
-### copyright(c) 2007-2008 kuwata-lab.com all rights reserved.
+### $Release: 0.7.0 $
+### copyright(c) 2007-2009 kuwata-lab.com all rights reserved.
 ###
 
 import unittest
-from test import test_support
 import sys, os, re
 
 from testcase_helper import *
@@ -16,7 +14,7 @@ from tenjin.helpers import *
 class TemplateTest(unittest.TestCase, TestCaseHelper):
 
     code = TestCaseHelper.generate_testcode(__file__)
-    exec code
+    exec(code)
 
 
     def setUp(self):
@@ -26,8 +24,6 @@ class TemplateTest(unittest.TestCase, TestCaseHelper):
         pass
 
     def _test(self):
-        if not self.is_target(depth=3): return
-        #
         input     = getattr(self, 'input', None)
         source    = getattr(self, 'source', None)
         expected  = getattr(self, 'expected', None)
@@ -45,7 +41,7 @@ class TemplateTest(unittest.TestCase, TestCaseHelper):
             return
         #
         if testopts:
-            if testopts.has_key('crchar'):
+            if 'crchar' in testopts:
                 ch = testopts['crchar']
                 if input:     input    = input.replace(ch, "\r")
                 if source:    source   = source.replace(ch, "\r")
@@ -55,18 +51,27 @@ class TemplateTest(unittest.TestCase, TestCaseHelper):
                 context['escape'] = cgi.escape
             if testopts.get('tostrfunc') == 'str':
                 context['to_str'] = str
-            if testopts.has_key('encoding'):
+            if 'encoding' in testopts:
                 encoding = testopts.get('encoding')
-            if testopts.has_key('templateclass'):
+            if 'templateclass' in testopts:
                 templateclass = testopts.get('templateclass')
         #
+        if python3:
+            input  = input.replace('urllib.quote', 'urllib.parse.quote')
+            source = source.replace('urllib.quote', 'urllib.parse.quote')
+            if encoding:
+                if source:
+                    source = source.replace("u'''", "'''").replace("u'", "'")
+                    input  = input.replace("u'", "'")
+        #
         if exception:
             try:
                 template = tenjin.Template(**options)
                 template.convert(input, filename)
                 template.render(context)
                 self.fail('%s is expected but not raised.' % exception)
-            except Exception, ex:
+            except Exception:
+                ex = sys.exc_info()[1]
                 self.assertEqual(exception, ex.__class__)
                 #self.assertTrue(isinstance(ex, exception))
                 if errormsg:
@@ -90,7 +95,6 @@ class TemplateTest(unittest.TestCase, TestCaseHelper):
 
 
 #    def test_render1(self):   # Tenjin#render(context) == Tenjin#render(**context)
-#        if not self.is_target(): return
 #        input = """<ul>
 #<?py for item in items: ?>
 #<li>#{item}</li>
@@ -107,16 +111,15 @@ class TemplateTest(unittest.TestCase, TestCaseHelper):
 
 
     def test_filename1(self):
-        if not self.is_target(): return
         input = """<ul>
-<?py for i in xrange(0,3): ?>
+<?py for i in range(0,3): ?>
 <li>#{i}</li>
 <?py #endfor ?>
 </ul>
 """
         filename = 'test_filename1.tenjin'
         try:
-            open(filename, 'w').write(input)
+            write_file(filename, input)
             template1 = tenjin.Template(filename)
             template2 = tenjin.Template()
             self.assertTextEqual(template1.script, template2.convert(input))
@@ -129,9 +132,11 @@ class TemplateTest(unittest.TestCase, TestCaseHelper):
 
 
     def test_import_module1(self):
-        if not self.is_target(): return
         import base64
-        input = "#{base64.encodestring('tenjin')}"
+        if python2:
+            input = "#{base64.encodestring('tenjin')}"
+        elif python3:
+            input = "#{base64.encodestring(b'tenjin')}"
         template = tenjin.Template()
         template.convert(input)
         def f1():
@@ -144,22 +149,27 @@ class TemplateTest(unittest.TestCase, TestCaseHelper):
 
 
     def test_import_module2(self):
-        if not self.is_target(): return
-        import rfc822
-        input = "#{rfc822.formatdate()}"
+        if python2:
+            import rfc822
+            input = "#{rfc822.formatdate()}"
+        elif python3:
+            import email.utils
+            input = "#{email.utils.formatdate()}"
         template = tenjin.Template()
         template.convert(input)
         def f1():
             template.render()
         self.assertRaises(NameError, f1)
-        #tenjin.import_module(rfc822)
-        globals()['rfc822'] = rfc822
+        if python2:
+            #tenjin.import_module(rfc822)
+            globals()['rfc822'] = rfc822
+        elif python3:
+            globals()['email'] = email
         #self.assertNotRaise(f1)
         f1()
 
 
     def test_invalid_template_args(self):
-        if not self.is_target(): return
         def f():
             input = "<?py #@ARGS 1x ?>"
             template = tenjin.Template()
@@ -167,16 +177,8 @@ class TemplateTest(unittest.TestCase, TestCaseHelper):
         self.assertRaises(ValueError, f)
 
 
-import os
-name = os.environ.get('TEST')
-if name:
-    for m in dir(TemplateTest):
-        if m.startswith('test_') and m != 'test_'+name:
-            delattr(TemplateTest, m)
-
-def test_main():
-    test_support.run_unittest(TemplateTest)
+remove_unmatched_test_methods(TemplateTest)
 
 
 if __name__ == '__main__':
-    test_main()
+    unittest.main()
